@@ -12,6 +12,7 @@ from livestyled.models import (
     Cohort,
     Competition,
     Device,
+    DeviceFormData,
     DevicePreference,
     DeviceReality,
     DeviceToken,
@@ -51,6 +52,7 @@ from livestyled.schemas import (
     BookingSchema,
     CohortSchema,
     CompetitionSchema,
+    DeviceFormDataSchema,
     DevicePreferenceSchema,
     DeviceRealitySchema,
     DeviceSchema,
@@ -1016,6 +1018,19 @@ class LiveStyledResourceClient(LiveStyledAPIClient):
     ) -> Venue:
         return self._get_resource_by_id(VenueSchema, id)
 
+    def update_venue(
+            self,
+            venue: Venue,
+            attributes: Dict
+    ) -> Venue:
+        return self._update_resource(VenueSchema, venue.id, attributes)
+
+    def create_venue(
+            self,
+            venue: Venue
+    ) -> Venue:
+        return self._create_resource(VenueSchema, venue)
+
     # ---- FULFILMENT POINTS
 
     def get_fulfilment_points(
@@ -1205,6 +1220,12 @@ class LiveStyledResourceClient(LiveStyledAPIClient):
             return self._get_resource_list(AudienceSchema, filters=filters)
         return self._get_resource_list(AudienceSchema)
 
+    def get_audiences_by_external_id(
+            self,
+            external_id: str,
+    ) -> Generator[Audience, None, None]:
+        return self._get_resource_list(AudienceSchema, external_id)
+
     # ---- AUDIENCE DEVICES
 
     def get_audience_devices(
@@ -1246,3 +1267,25 @@ class LiveStyledResourceClient(LiveStyledAPIClient):
             self,
     ) -> Generator[Location, None, None]:
         return self._get_resource_list(LocationSchema)
+
+    # ---- FORM_DATA
+
+    def get_form_data(
+            self,
+            device_id=None,
+            form_id=None,
+            expires_at_before=None,
+            expires_at_after=None
+    ) -> Generator[DeviceFormData, None, None]:
+        filters = {}
+        if device_id:
+            filters['device'] = device_id
+        if form_id:
+            filters['form'] = form_id
+        if expires_at_before:
+            filters['expiresAt[before]'] = expires_at_before
+        if expires_at_after:
+            filters['expiresAt[after]'] = expires_at_after
+        if filters:
+            return self._get_resource_list(DeviceFormDataSchema, filters=filters)
+        return self._get_resource_list(DeviceFormDataSchema)
